@@ -1,24 +1,32 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.directives
     ~~~~~~~~~~~~~~~~~
 
     Handlers for additional ReST directives.
 
-    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
+from typing import Any, Dict, List, Tuple
+from typing import cast
 
-from docutils.parsers.rst import Directive, directives
+from docutils import nodes
+from docutils.nodes import Node
+from docutils.parsers.rst import directives, roles
 
 from sphinx import addnodes
-from sphinx.util.docfields import DocFieldTransformer
+from sphinx.addnodes import desc_signature
+from sphinx.deprecation import RemovedInSphinx40Warning, deprecated_alias
+from sphinx.util import docutils
+from sphinx.util.docfields import DocFieldTransformer, Field, TypedField
+from sphinx.util.docutils import SphinxDirective
+from sphinx.util.typing import DirectiveOption
 
-# import and register directives
-from sphinx.directives.code import *
-from sphinx.directives.other import *
+if False:
+    # For type annotation
+    from sphinx.application import Sphinx
 
 
 # RE to strip backslash escapes
@@ -26,7 +34,20 @@ nl_escape_re = re.compile(r'\\\n')
 strip_backslash_re = re.compile(r'\\(.)')
 
 
-class ObjectDescription(Directive):
+def optional_int(argument):
+    """
+    Check for an integer argument or None value; raise ``ValueError`` if not.
+    """
+    if argument is None:
+        return None
+    else:
+        value = int(argument)
+        if value < 0:
+            raise ValueError('negative value; must be positive or zero')
+        return value
+
+
+class ObjectDescription(SphinxDirective):
     """
     Directive to describe a class, function or similar object.  Not used
     directly, but subclassed (in domain-specific directives) to add custom
@@ -39,12 +60,32 @@ class ObjectDescription(Directive):
     final_argument_whitespace = True
     option_spec = {
         'noindex': directives.flag,
-    }
+    }  # type: Dict[str, DirectiveOption]
 
     # types of doc fields that this directive handles, see sphinx.util.docfields
-    doc_field_types = []
+    doc_field_types = []    # type: List[Field]
+    domain = None           # type: str
+    objtype = None          # type: str
+    indexnode = None        # type: addnodes.index
 
-    def get_signatures(self):
+    # Warning: this might be removed in future version. Don't touch this from extensions.
+    _doc_field_type_map = {}  # type: Dict[str, Tuple[Field, bool]]
+
+    def get_field_type_map(self) -> Dict[str, Tuple[Field, bool]]:
+        if self._doc_field_type_map == {}:
+            self._doc_field_type_map = {}
+            for field in self.doc_field_types:
+                for name in field.names:
+                    self._doc_field_type_map[name] = (field, False)
+
+                if field.is_typed:
+                    typed_field = cast(TypedField, field)
+                    for name in typed_field.typenames:
+                        self._doc_field_type_map[name] = (field, True)
+
+        return self._doc_field_type_map
+
+    def get_signatures(self) -> List[str]:
         """
         Retrieve the signatures to document from the directive arguments.  By
         default, signatures are given as arguments, one per line.
@@ -55,7 +96,7 @@ class ObjectDescription(Directive):
         # remove backslashes to support (dummy) escapes; helps Vim highlighting
         return [strip_backslash_re.sub(r'\1', line.strip()) for line in lines]
 
-    def handle_signature(self, sig, signode):
+    def handle_signature(self, sig: str, signode: desc_signature) -> Any:
         """
         Parse the signature *sig* into individual nodes and append them to
         *signode*. If ValueError is raised, parsing is aborted and the whole
@@ -67,7 +108,7 @@ class ObjectDescription(Directive):
         """
         raise ValueError
 
-    def add_target_and_index(self, name, sig, signode):
+    def add_target_and_index(self, name: Any, sig: str, signode: desc_signature) -> None:
         """
         Add cross-reference IDs and entries to self.indexnode, if applicable.
 
@@ -75,21 +116,21 @@ class ObjectDescription(Directive):
         """
         return  # do nothing by default
 
-    def before_content(self):
+    def before_content(self) -> None:
         """
         Called before parsing content. Used to set information about the current
         directive context on the build environment.
         """
         pass
 
-    def after_content(self):
+    def after_content(self) -> None:
         """
         Called after parsing content. Used to reset information about the
         current directive context on the build environment.
         """
         pass
 
-    def run(self):
+    def run(self) -> List[Node]:
         """
         Main directive entry function, called by docutils upon encountering the
         directive.
@@ -111,7 +152,6 @@ class ObjectDescription(Directive):
             self.domain, self.objtype = self.name.split(':', 1)
         else:
             self.domain, self.objtype = '', self.name
-        self.env = self.state.document.settings.env
         self.indexnode = addnodes.index(entries=[])
 
         node = addnodes.desc()
@@ -121,7 +161,7 @@ class ObjectDescription(Directive):
         node['objtype'] = node['desctype'] = self.objtype
         node['noindex'] = noindex = ('noindex' in self.options)
 
-        self.names = []
+        self.names = []  # type: List[Any]
         signatures = self.get_signatures()
         for i, sig in enumerate(signatures):
             # add a signature node for each signature in the current unit
@@ -158,11 +198,36 @@ class ObjectDescription(Directive):
         self.after_content()
         return [self.indexnode, node]
 
-# backwards compatible old name
-DescDirective = ObjectDescription
+
+class DefaultRole(SphinxDirective):
+    """
+    Set the default interpreted text role.  Overridden from docutils.
+    """
+
+    optional_arguments = 1
+    final_argument_whitespace = False
+
+    def run(self) -> List[Node]:
+        if not self.arguments:
+            docutils.unregister_role('')
+            return []
+        role_name = self.arguments[0]
+        role, messages = roles.role(role_name, self.state_machine.language,
+                                    self.lineno, self.state.reporter)
+        if role:
+            docutils.register_role('', role)
+            self.env.temp_data['default_role'] = role_name
+        else:
+            literal_block = nodes.literal_block(self.block_text, self.block_text)
+            reporter = self.state.reporter
+            error = reporter.error('Unknown interpreted text role "%s".' % role_name,
+                                   literal_block, line=self.lineno)
+            messages += [error]
+
+        return cast(List[nodes.Node], messages)
 
 
-class DefaultDomain(Directive):
+class DefaultDomain(SphinxDirective):
     """
     Directive to (re-)set the default domain for this source file.
     """
@@ -171,22 +236,66 @@ class DefaultDomain(Directive):
     required_arguments = 1
     optional_arguments = 0
     final_argument_whitespace = False
-    option_spec = {}
+    option_spec = {}  # type: Dict
 
-    def run(self):
-        env = self.state.document.settings.env
+    def run(self) -> List[Node]:
         domain_name = self.arguments[0].lower()
         # if domain_name not in env.domains:
         #     # try searching by label
-        #     for domain in env.domains.itervalues():
+        #     for domain in env.domains.values():
         #         if domain.label.lower() == domain_name:
         #             domain_name = domain.name
         #             break
-        env.temp_data['default_domain'] = env.domains.get(domain_name)
+        self.env.temp_data['default_domain'] = self.env.domains.get(domain_name)
         return []
 
+from sphinx.directives.code import (  # noqa
+    Highlight, CodeBlock, LiteralInclude
+)
+from sphinx.directives.other import (  # noqa
+    TocTree, Author, Index, VersionChange, SeeAlso,
+    TabularColumns, Centered, Acks, HList, Only, Include, Class
+)
+from sphinx.directives.patches import (  # noqa
+    Figure, Meta
+)
 
-directives.register_directive('default-domain', DefaultDomain)
-directives.register_directive('describe', ObjectDescription)
-# new, more consistent, name
-directives.register_directive('object', ObjectDescription)
+deprecated_alias('sphinx.directives',
+                 {
+                     'Highlight': Highlight,
+                     'CodeBlock': CodeBlock,
+                     'LiteralInclude': LiteralInclude,
+                     'TocTree': TocTree,
+                     'Author': Author,
+                     'Index': Index,
+                     'VersionChange': VersionChange,
+                     'SeeAlso': SeeAlso,
+                     'TabularColumns': TabularColumns,
+                     'Centered': Centered,
+                     'Acks': Acks,
+                     'HList': HList,
+                     'Only': Only,
+                     'Include': Include,
+                     'Class': Class,
+                     'Figure': Figure,
+                     'Meta': Meta,
+                 },
+                 RemovedInSphinx40Warning)
+
+
+# backwards compatible old name (will be marked deprecated in 3.0)
+DescDirective = ObjectDescription
+
+
+def setup(app: "Sphinx") -> Dict[str, Any]:
+    directives.register_directive('default-role', DefaultRole)
+    directives.register_directive('default-domain', DefaultDomain)
+    directives.register_directive('describe', ObjectDescription)
+    # new, more consistent, name
+    directives.register_directive('object', ObjectDescription)
+
+    return {
+        'version': 'builtin',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
